@@ -458,12 +458,15 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 
     new_doc = xmlNewDoc ("1.0");
     new_doc->charset = XML_CHAR_ENCODING_UTF8;
+    new_doc->dict = ctxt->dict;
+    xmlDictReference (new_doc->dict);
+
     ctxt->output = new_doc;
     ctxt->insert = (xmlNodePtr) new_doc;
 
     xsltApplyOneTemplate (ctxt, node, inst->children, NULL, NULL);
 
-    xmlDocDumpFormatMemory (new_doc, &page_buf, &buf_size, 0);
+    xsltSaveResultToString (&page_buf, &buf_size, new_doc, style);
 
     ctxt->outputFile = old_outfile;
     ctxt->output     = old_doc;
@@ -508,6 +511,7 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 	gtk_main_iteration ();
 
     xmlFreeDoc (new_doc);
+    xsltFreeStylesheet (style);
 }
 
 void
@@ -549,13 +553,14 @@ walker_walk_xml (DBWalker *walker)
 
     id = xmlGetProp (walker->cur, "id");
     if (!id && walker->cur->parent->type == XML_DOCUMENT_NODE) {
-	id = g_strdup ("index");
+	id = xmlStrdup ("index");
     }
 
     if (walker_is_chunk (walker)) {
 	if (xml_is_info (walker->cur)) {
-	    xmlFree (id);
-	    id = g_strdup ("titlepage");
+	    if (id)
+		xmlFree (id);
+	    id = xmlStrdup ("titlepage");
 	}
 
 	title = xml_get_title (walker->cur);
@@ -616,8 +621,10 @@ walker_walk_xml (DBWalker *walker)
 	walker->page_id = old_id;
     }
 
-    xmlFree (id);
-    xmlFree (title);
+    if (id != NULL)
+	xmlFree (id);
+    if (title != NULL)
+	xmlFree (title);
 }
 
 gchar *
